{--------------------------------------------------------------------

 Author:      J.M.Wehlou

 Description: This unit implements performance counting for the client
              app and needs to be customized for the application.

 --------------------------------------------------------------------}

unit PerfMonClientImpl;

interface

type
  IPerfMonClientImpl = interface
    procedure Install();
    procedure Uninstall();
    function  IsInstalled(): boolean;
  end;

  function getPerfMonClientImpl(): IPerfMonClientImpl;

// ==================================================================

implementation

// You have to include all monitored units in the uses clause

uses
  PerfMonLib,
  PerfMonClientDef,
  ClientMonitoredUnit;

type
  (*
     The class implements its "own" interface, plus the callback
     interface for each monitored unit. In this case, there is only
     one monitored unit (ClientMonitoredUnit), but you could
     have any number here.

     Even though this class implements a number of interfaces, it is
     not reference counted, so I have to destroy it explicitly in
     the finalizer of the unit (see bottom of the unit). To stop it
     from being reference counted, I override _AddRef() and _Release()
     and let them do precisely nothing.
   *)

  TPerfMonClientImpl = class(TInterfacedObject, IPerfMonClientImpl,
                              IPerfMonClientCb,
                              IClientMonitoredPerf)
    fPML          : IPerfMonLib;
    fuDeltaCount  : cardinal;
  public
    constructor Create();
    destructor  Destroy; override;
    procedure   ConnectSources();
    procedure   DisconnectSources();
    procedure   PmcSet(pmc: pmcCounter; uVal: cardinal);
    // IPerfMonClientCb
    procedure PerfMonIsRunning(bRunning: Boolean);
    // IPerfMonClientImpl
    procedure   Install;
    function    IsInstalled: Boolean;
    procedure   Uninstall;
    // IClientMonitoredPerf
    procedure   CMDeltaCount(uVal: Cardinal);
    procedure   CMRawCount(uVal: Cardinal);
  private
    function    _AddRef(): integer; stdcall;
    function    _Release(): integer; stdcall;
  end;

var
  // this variable has to be of class type, not interface, else
  // I can't "Free" it in the finalization section
  gPMCI   : TPerfMonClientImpl = nil;

// ==================================================================

// the TPerfMonClientImpl is a "lazy" singleton, so there is only one
// instance and it's only created if anyone asks for it.

function getPerfMonClientImpl(): IPerfMonClientImpl;
begin
  if not assigned(gPMCI) then
    gPMCI := TPerfMonClientImpl.Create();
  Result := gPMCI;
end;

// ==================================================================
{ TPerfMonClientImpl }

// callback from ClientMonitoredUnit
procedure TPerfMonClientImpl.CMDeltaCount(uVal: Cardinal);
begin
  // The Windows Performance Monitor needs to get a total count. It
  // calculates the delta per time unit itself.
  Inc(fuDeltaCount, uVal);
  PmcSet(pmcDeltaCounter, fuDeltaCount);
end;

// ------------------------------------------------------------------

// callback from ClientMonitoredUnit
procedure TPerfMonClientImpl.CMRawCount(uVal: Cardinal);
begin
  // Raw counts go up and down and should be given unchanged
  // to the Windows Performance Monitor
  PmcSet(pmcRawCounter, uVal);
end;

// ------------------------------------------------------------------

procedure TPerfMonClientImpl.ConnectSources;
begin
  // Tell the monitored units to call back with performance data.
  // Do one call for each monitored unit.
  setClientMonitoredPerf(self);
end;

// ------------------------------------------------------------------

constructor TPerfMonClientImpl.Create;
begin
  { when creating an instance of the performance monitor handler,
    give it a pointer to this unit, but also a pointer to the
    unit that has all the constants and settings for this application }
  fPML := getPerfMonLib(getPerfMonClientDef(), self);
end;

// ------------------------------------------------------------------

destructor TPerfMonClientImpl.Destroy;
begin
  DisconnectSources();
  fPML := nil;
  inherited;
end;

// ------------------------------------------------------------------

procedure TPerfMonClientImpl.DisconnectSources;
begin
  // Unhook all monitored units, so they don't call back to
  // a non-existant object after the destruction.
  setClientMonitoredPerf(nil);
end;

// ------------------------------------------------------------------

procedure TPerfMonClientImpl.Install;
begin
  fPML.InstallPerfMon();
end;

// ------------------------------------------------------------------

function TPerfMonClientImpl.IsInstalled: Boolean;
begin
  Result := assigned(fPML) and fPML.IsInstalled();
end;

// ------------------------------------------------------------------

{ called from PerfMonLib as it starts and stops, so here
  I tell the different monitored units where to send their counter
  values, that is to this unit }
procedure TPerfMonClientImpl.PerfMonIsRunning(bRunning: Boolean);
begin
  if bRunning then
    ConnectSources()
  else
    DisconnectSources();
end;

// ------------------------------------------------------------------

procedure TPerfMonClientImpl.PmcSet(pmc: pmcCounter; uVal: cardinal);
var
  uOffs   : cardinal;
begin
  uOffs := Ord(pmc) * sizeof(cardinal);
  fPML.SetValue(uOffs, uVal);
end;

// ------------------------------------------------------------------

procedure TPerfMonClientImpl.Uninstall;
begin
  fPML.UninstallPerfMon();
end;

// ------------------------------------------------------------------

function TPerfMonClientImpl._AddRef: integer;
begin
  Result := -1;
end;

// ------------------------------------------------------------------

function TPerfMonClientImpl._Release: integer;
begin
  Result := -1;
end;

// ==================================================================

initialization

finalization
  // if it's not assigned, nothing happens
  gPMCI.Free;

end.
